@@ -13,6 +13,12 @@ the use case is correct.
 void cem_add_to_log(char *message,int type){
   if (type==LOG_DEBUG) {
     printf("**DEBUG: %s **\n",message);
+  }else if(type==LOG_ERROR){
+    printf("**ERROR: %s **\n",message);
+  }else if(type==LOG_NETWORK_DEBUG){
+  	printf("**NETWORK DEBUG: %s**\n",message);
+  }else if(type==LOG_NETWORK){
+	  printf("**NETWORKING: %s**\n",message);
   }else{
     printf("**MESSAGE: %s **\n",message);
   }
