@@ -26,10 +26,12 @@ struct _JammoMentorPrivate {
 	gfloat standby_scale;
 	gfloat active_scale;
 	gchar* idle_speech;
+	JammoMentorSpokenCallback idle_speech_spoken_callback;
+	gpointer idle_speech_spoken_user_data;
 	gulong idle_timeout;
+	gchar* language;
 	
 	GList* spoken_speeches;
-	ClutterScript* script;
 	JammoSample* active_sample;
 
 	ClutterActor* stage;
@@ -38,9 +40,10 @@ struct _JammoMentorPrivate {
 	time_t last_activity_time;
 	
 	guint idle_speech_spoken : 1;
+	guint interrupted : 1;
 };
 
-static void speak(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCallback callback, gpointer user_data);
+static void speak(JammoMentor* mentor, const gchar* speech, gboolean highlight, gfloat offset_x, gfloat offset_y, JammoMentorSpokenCallback callback, gpointer user_data);
 static void set_idle_timeout(JammoMentor* mentor, gboolean activate);
 
 static GList* mentors;
@@ -81,29 +84,52 @@ void jammo_mentor_set_active_scale(JammoMentor* mentor, gfloat scale) {
 }
 
 void jammo_mentor_speak(JammoMentor* mentor, const gchar* speech) {
-	speak(mentor, speech, NULL, NULL);
+	speak(mentor, speech, FALSE, 0.0, 0.0, NULL, NULL);
 }
 
 void jammo_mentor_speak_with_callback(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCallback callback, gpointer user_data) {
-	speak(mentor, speech, callback, user_data);
+	speak(mentor, speech, FALSE, 0.0, 0.0, callback, user_data);
+}
+
+void jammo_mentor_speak_and_highlight(JammoMentor* mentor, const gchar* speech, gfloat offset_x, gfloat offset_y) {
+	speak(mentor, speech, TRUE, offset_x, offset_y, NULL, NULL);
+}
+
+void jammo_mentor_speak_and_highlight_with_callback(JammoMentor* mentor, const gchar* speech, gfloat offset_x, gfloat offset_y, JammoMentorSpokenCallback callback, gpointer user_data) {
+	speak(mentor, speech, TRUE, offset_x, offset_y, callback, user_data);
 }
 
 void jammo_mentor_speak_once(JammoMentor* mentor, const gchar* speech) {
 	if (!g_list_find_custom(mentor->priv->spoken_speeches, speech, (GCompareFunc)g_strcmp0)) {
 		mentor->priv->spoken_speeches = g_list_prepend(mentor->priv->spoken_speeches, g_strdup(speech));
-		speak(mentor, speech, NULL, NULL);
+		speak(mentor, speech, FALSE, 0.0, 0.0, NULL, NULL);
 	}
 }
 
 void jammo_mentor_speak_once_with_callback(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCallback callback, gpointer user_data) {
 	if (!g_list_find_custom(mentor->priv->spoken_speeches, speech, (GCompareFunc)g_strcmp0)) {
 		mentor->priv->spoken_speeches = g_list_prepend(mentor->priv->spoken_speeches, g_strdup(speech));
-		speak(mentor, speech, callback, user_data);
+		speak(mentor, speech, FALSE, 0.0, 0.0, callback, user_data);
+	}
+}
+
+void jammo_mentor_speak_and_highlight_once(JammoMentor* mentor, const gchar* speech, gfloat offset_x, gfloat offset_y) {
+	if (!g_list_find_custom(mentor->priv->spoken_speeches, speech, (GCompareFunc)g_strcmp0)) {
+		mentor->priv->spoken_speeches = g_list_prepend(mentor->priv->spoken_speeches, g_strdup(speech));
+		speak(mentor, speech, TRUE, offset_x, offset_y, NULL, NULL);
+	}
+}
+
+void jammo_mentor_speak_and_highlight_once_with_callback(JammoMentor* mentor, const gchar* speech, gfloat offset_x, gfloat offset_y, JammoMentorSpokenCallback callback, gpointer user_data) {
+	if (!g_list_find_custom(mentor->priv->spoken_speeches, speech, (GCompareFunc)g_strcmp0)) {
+		mentor->priv->spoken_speeches = g_list_prepend(mentor->priv->spoken_speeches, g_strdup(speech));
+		speak(mentor, speech, TRUE, offset_x, offset_y, callback, user_data);
 	}
 }
 
 void jammo_mentor_shut_up(JammoMentor* mentor) {
 	if (mentor->priv->active_sample) {
+		mentor->priv->interrupted = TRUE;
 		jammo_sample_stop(mentor->priv->active_sample);
 	}
 }
@@ -114,6 +140,10 @@ const gchar* jammo_mentor_get_idle_speech(JammoMentor* mentor) {
 }
 
 void jammo_mentor_set_idle_speech(JammoMentor* mentor, const gchar* speech) {
+	jammo_mentor_set_idle_speech_with_callback(mentor, speech, NULL, NULL);
+}
+
+void jammo_mentor_set_idle_speech_with_callback(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCallback callback, gpointer user_data) {
 	if (g_strcmp0(mentor->priv->idle_speech, speech)) {
 		g_free(mentor->priv->idle_speech);
 		mentor->priv->idle_speech = g_strdup(speech);
@@ -121,6 +151,8 @@ void jammo_mentor_set_idle_speech(JammoMentor* mentor, const gchar* speech) {
 		set_idle_timeout(mentor, TRUE);
 		g_object_notify(G_OBJECT(mentor), "idle-speech");
 	}
+	mentor->priv->idle_speech_spoken_callback = callback;
+	mentor->priv->idle_speech_spoken_user_data = user_data;
 }
 
 gulong jammo_mentor_get_idle_timeout(JammoMentor* mentor) {
@@ -136,6 +168,16 @@ void jammo_mentor_set_idle_timeout(JammoMentor* mentor, gulong milliseconds) {
 	}
 }
 
+const gchar* jammo_mentor_get_language(JammoMentor* mentor) {
+
+	return mentor->priv->language;
+}
+
+void jammo_mentor_set_language(JammoMentor* mentor, const gchar* language) {
+	g_free(mentor->priv->language);
+	mentor->priv->language = g_strdup(language);
+}
+
 JammoMentor* jammo_mentor_get_default(void) {
 
 	return (mentors ? JAMMO_MENTOR(mentors->data) : NULL);
@@ -147,9 +189,10 @@ static gboolean jammo_mentor_clicked(TangleButton* button) {
 	mentor = JAMMO_MENTOR(button);
 	
 	if (mentor->priv->active_sample) {
+		mentor->priv->interrupted = TRUE;
 		jammo_sample_stop(mentor->priv->active_sample);
 	} else if (mentor->priv->idle_speech) {
-		speak(mentor, mentor->priv->idle_speech, NULL, NULL);
+		speak(mentor, mentor->priv->idle_speech, FALSE, 0.0, 0.0, mentor->priv->idle_speech_spoken_callback, mentor->priv->idle_speech_spoken_user_data);
 	}
 
 	return FALSE;
@@ -308,18 +351,21 @@ static void on_sample_stopped(JammoSample* sample, gpointer user_data) {
 	g_object_unref(mentor->priv->active_sample);
 	mentor->priv->active_sample = NULL;
 
-	tangle_actor_animate(TANGLE_ACTOR(mentor), CLUTTER_EASE_IN_QUAD, 1000, "scale-x", mentor->priv->standby_scale, "scale-y", mentor->priv->standby_scale, NULL);
+	tangle_actor_animate(TANGLE_ACTOR(mentor), CLUTTER_EASE_IN_QUAD, 1000, "squint-x", 0.0, "squint-y", 0.0, "scale-x", mentor->priv->standby_scale, "scale-y", mentor->priv->standby_scale, NULL);
 
 	set_idle_timeout(mentor, TRUE);
 	
 	if (callback) {
-		callback(mentor, speech, data);
+		callback(mentor, speech, mentor->priv->interrupted, data);
 	}
+
+	mentor->priv->interrupted = FALSE;
 	
-	g_free(speech);
+	/* TODO: g_free(speech); ? */
 }
 
-static void speak(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCallback callback, gpointer user_data) {
+static void speak(JammoMentor* mentor, const gchar* speech, gboolean highlight, gfloat offset_x, gfloat offset_y, JammoMentorSpokenCallback callback, gpointer user_data) {
+	gchar* localized_speech;
 	gchar* filename;
 	TangleVault* vault;
 	
@@ -330,18 +376,24 @@ static void speak(JammoMentor* mentor, const gchar* speech, JammoMentorSpokenCal
 		jammo_sample_stop(mentor->priv->active_sample);
 	}
 	
-	if (mentor->priv->script && (filename = clutter_script_lookup_filename(mentor->priv->script, speech))) {
+	if (speech[0] == '/' || !mentor->priv->language) {
+		localized_speech = g_strdup(speech);
+	} else {
+		localized_speech = g_strconcat(mentor->priv->language, "/", speech, NULL);
+	}
+	if ((filename = tangle_actor_lookup_filename(TANGLE_ACTOR(mentor), localized_speech))) {
 		mentor->priv->active_sample = jammo_sample_new_from_file(filename);
 		g_free(filename);
 	} else {
-		mentor->priv->active_sample = jammo_sample_new_from_file(speech);
+		mentor->priv->active_sample = jammo_sample_new_from_file(localized_speech);
 	}
+	g_free(localized_speech);
 	
 	vault = tangle_vault_new(4, JAMMO_TYPE_MENTOR, mentor, G_TYPE_POINTER, callback, G_TYPE_STRING, speech, G_TYPE_POINTER, user_data);
 	tangle_signal_connect_vault(mentor->priv->active_sample, "stopped", G_CALLBACK(on_sample_stopped), vault);
 	
-	tangle_actor_animate(TANGLE_ACTOR(mentor), CLUTTER_EASE_IN_QUAD, 1000, "scale-x", mentor->priv->active_scale, "scale-y", mentor->priv->active_scale, NULL);
-	
+	tangle_actor_animate(TANGLE_ACTOR(mentor), CLUTTER_EASE_IN_QUAD, 1000, "squint-x", offset_x, "squint-y", offset_y, "scale-x", mentor->priv->active_scale, "scale-y", mentor->priv->active_scale, NULL);
+		
 	jammo_sample_play(mentor->priv->active_sample);
 }
 
@@ -365,7 +417,7 @@ static gboolean on_timeout(gpointer user_data) {
 	if (elapsed_time >= mentor->priv->idle_timeout) {
 		mentor->priv->timeout_event_source_id = 0;
 		mentor->priv->idle_speech_spoken = TRUE;
-		speak(mentor, mentor->priv->idle_speech, NULL, NULL);
+		speak(mentor, mentor->priv->idle_speech, FALSE, 0.0, 0.0, mentor->priv->idle_speech_spoken_callback, mentor->priv->idle_speech_spoken_user_data);
 	} else {
 		mentor->priv->timeout_event_source_id = g_timeout_add((mentor->priv->idle_timeout - elapsed_time) * 1000, on_timeout, mentor);
 	}
