@@ -14,6 +14,7 @@
 #include <tangle.h>
 #include <clutter/clutter.h>
 #include <stdlib.h> 
+#include <libgen.h>
 
 #include "jammo-mentor.h"
 
@@ -191,6 +192,8 @@ int main(int argc, char** argv) {
 	GObject* object;
 	ClutterActor* actor;
 	ClutterColor black = { 0, 0, 0, 255 };
+	gchar* application_path;
+	const gchar* application_name;
 
 	jammo_meam_init(&argc, &argv);
 	jammo_chum_init(&argc, &argv);
@@ -298,13 +301,15 @@ int main(int argc, char** argv) {
 
 
 
-	//printf("Application name is '%s'\n",argv[0]);
-	if (strcmp(argv[0],"./jammo3_6")==0) { //start jammo 3-6 years
+	application_path = g_strdup(argv[0]);
+	application_name = basename(application_path);
+	
+	if (!strcmp(application_name,"jammo3_6")) { //start jammo 3-6 years
 		printf("Starting 3-6 years game\n");
 		g_timeout_add(10000, door_timeouts,NULL); //10 seconds and door will be opened automatically
 	}
 
-	else if (strcmp(argv[0],"./jammo7_12")==0) { //start jammo 7-12 years
+	else if (!strcmp(application_name,"jammo7_12")) { //start jammo 7-12 years
 		printf("Starting 7-12 years game\n");
 		//start_gamesmenu(); //this only starts mockup menu
 		//start_sequencer(NULL); //this starts empty sequencer
@@ -339,6 +344,7 @@ int main(int argc, char** argv) {
 		}
 	}
 
+	g_free(application_path);
 
 	clutter_main();
 
