@@ -26,6 +26,7 @@
 #include "groupmanager.h"
 #include "../cem/cem.h"
 #include "gems_service_collaboration.h"
+#include "gems_networking_utils.h"
 
 #define SOCKET_ERROR        -1
 #define HOST_NAME_SIZE      255
@@ -358,6 +359,16 @@ void gems_communication_cleanup()
 	
 	if(data == NULL) return;
 	
+	gems_communication_cleanup_lists();
+	
+	g_free(data);
+	data = NULL;
+}
+
+void gems_communication_cleanup_lists()
+{
+	gems_communication_data* data = gems_get_data()->communication;
+	
 	if(data->connections != NULL)
 	{
 		g_list_foreach(data->connections, (GFunc)gems_clear_gems_connection, NULL);
@@ -378,9 +389,6 @@ void gems_communication_cleanup()
 		g_list_free(data->rejected);
 		data->rejected = NULL;
 	}
-	
-	g_free(data);
-	data = NULL;
 }
 
 gint gems_communication_search_jammos()
@@ -395,10 +403,26 @@ gint gems_communication_search_jammos()
 	// Get surrounding devices with service JAMMO_SERVICE_NAME 
 	TDeviceList* list = ph_c_get_devicelist_with_services(data->ph,JAMMO_SERVICE_NAME);
 	
+	// Got a NULL reference - error with PeerHood -> stop it
+	if(list == NULL)
+	{
+		logmsg = g_strdup_printf("gems_communication_search_jammos: connection to PeerHood daemon was lost.");
+		cem_add_to_log(logmsg, LOG_NETWORK_DEBUG);
+		g_free(logmsg);
+		gems_communication_peerhood_close_detected();
+		return FALSE;
+	}
+	
 	if(ph_c_devicelist_is_empty(list) == FALSE)
 	{
 		// Initialize wrapper for iterator for list
 		DeviceIterator* iterator = ph_c_new_device_iterator(list);
+		
+		if(iterator == NULL)
+		{
+			ph_c_delete_devicelist(list);
+			return TRUE;
+		}
 
 		// Go through list
 		for(iterator = ph_c_new_device_iterator(list); ph_c_device_iterator_is_last(iterator,list) != TRUE; ph_c_device_iterator_next_iterator(iterator))
@@ -581,11 +605,27 @@ gint gems_communication_request_groups()
 	// Get surrounding devices with service GROUP_SERVICE_NAME 
 	TDeviceList* list = ph_c_get_devicelist_with_services(data->ph,GROUP_SERVICE_NAME);
 	
+	// Got a NULL reference - error with PeerHood -> stop it
+	if(list == NULL)
+	{
+		logmsg = g_strdup_printf("gems_communication_request_groups: connection to PeerHood daemon was lost.");
+		cem_add_to_log(logmsg, LOG_NETWORK_DEBUG);
+		g_free(logmsg);
+		gems_communication_peerhood_close_detected();
+		return FALSE;
+	}
+	
 	if(ph_c_devicelist_is_empty(list) == FALSE)
 	{
 		// Initialize wrapper for iterator for list
 		DeviceIterator* iterator = NULL;
 
+		if(iterator == NULL)
+		{
+			ph_c_delete_devicelist(list);
+			return TRUE;
+		}
+		
 		// Go through list
 		for(iterator = ph_c_new_device_iterator(list); ph_c_device_iterator_is_last(iterator,list) != TRUE; ph_c_device_iterator_next_iterator(iterator))
 		{
@@ -1676,3 +1716,106 @@ void gems_communication_process_error(gems_connection* element)
 	}
 }
 
+
+gint gems_communication_probe_for_peerhood()
+{
+	gems_components* data = gems_get_data();
+	
+	if(data == NULL) return FALSE;
+	
+	gchar* logmsg = g_strdup_printf("gems_networking_probe_for_peerhood");
+	cem_add_to_log(logmsg,LOG_INFO);
+	g_free(logmsg);
+	
+	// Initialize callback if it was not set (PeerHood not found at start)
+	if(data->ph_cb == NULL) gems_init_peerhood_callback();
+	
+	// Get PeerHood instance if not set
+	if(data->ph == NULL) data->ph = ph_c_get_instance(data->ph_cb);
+	
+	// Cannot get instance - should not happen
+	if(data->ph == NULL)
+	{
+		logmsg = g_strdup_printf("gems_networking_probe_for_peerhood: Got PeerHood NULL reference.");
+		cem_add_to_log(logmsg,LOG_ERROR);
+		g_free(logmsg);
+		return TRUE; // Got NULL reference, try again
+	}
+	
+	gchar** args = (gchar**)g_malloc(sizeof(gchar**));
+	args[0] = "./jammo";
+	if(ph_c_init(data->ph,1,args) == TRUE)
+	{
+		gems_register_services();
+		gems_enable_network_timeout_functions();
+		logmsg = g_strdup_printf("gems_networking_probe_for_peerhood: initialized and re-enabled timeout functions.");
+		cem_add_to_log(logmsg,LOG_INFO);
+		g_free(logmsg);
+		g_free(args);
+		return FALSE; // Got connection, remove
+	}
+	else
+	{
+		logmsg = g_strdup_printf("gems_networking_probe_for_peerhood: cannot initialize peerhood.");
+		cem_add_to_log(logmsg,LOG_INFO);
+		g_free(logmsg);
+		g_free(args);
+		return TRUE;
+	}
+}
+
+
+void gems_communication_peerhood_close_detected()
+{
+	gchar* logmsg = NULL;
+	gems_components* data = gems_get_data();
+	
+	if(data == NULL) return;
+	
+	logmsg = g_strdup_printf("gems_networking_peerhood_close_detected.");
+	cem_add_to_log(logmsg,LOG_INFO);
+	g_free(logmsg);
+	
+	if(data->ph == NULL)
+	{
+		logmsg = g_strdup_printf("gems_networking_peerhood_close_detected: PeerHood NULL.");
+		cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+		g_free(logmsg);
+	}
+	else
+	{
+		ph_c_delete_peerhood(data->ph);
+		data->ph = NULL;
+		
+		//ph_c_delete_callback(data->ph_cb);
+		//data->ph_cb = NULL;
+	}
+	
+	data->service_jammo->enabled = FALSE;
+	data->service_profile->enabled = FALSE;
+	data->service_group->enabled = FALSE;
+	data->service_collaboration->enabled = FALSE;
+		
+	
+	/*jammo_gems_cleanup_lists();
+	printf("jammo_gems_cleanup_lists\n");
+
+	gems_communication_cleanup_lists();
+	printf("gems_networking_cleanup_lists\n");*/
+	
+	// TODO create and user cleanup_lists() of each service
+	/*gems_service_jammo_cleanup_lists();
+	gems_service_profile_cleanup_lists();
+	gems_service_group_cleanup_lists();
+	gems_service_collaboration_cleanup_lists();
+	
+	*/
+	/*
+	g_timeout_add_full(G_PRIORITY_LOW,2000,(GSourceFunc)gems_communication_probe_for_peerhood, NULL, NULL);
+	logmsg = g_strdup_printf("gems_networking_peerhood_close_detected: deleted, timeout for probe added.");
+	cem_add_to_log(logmsg,LOG_INFO);
+	g_free(logmsg);*/
+	
+}
+
+
