@@ -175,6 +175,7 @@ void jammo_gems_init(int argc, char* argv[])
 		components->service_jammo->enabled = FALSE;
 		components->service_profile->enabled = FALSE;
 		components->service_group->enabled = FALSE;
+		components->service_collaboration->enabled = FALSE;
 		//gems_service_jammo_cleanup(components->service_jammo);
 		
 		//gems_communication_cleanup(components->communication);
@@ -184,6 +185,8 @@ void jammo_gems_init(int argc, char* argv[])
 		
 		ph_c_delete_callback(components->ph_cb);
 		components->ph_cb = NULL;
+		
+		//g_timeout_add_full(G_PRIORITY_LOW,2000,(GSourceFunc)gems_communication_probe_for_peerhood, NULL, NULL);
 	}
 	
 	components->ip_data = (ip_communication*)g_malloc(sizeof(ip_communication));
@@ -230,15 +233,8 @@ void jammo_gems_cleanup()
 	
 	if(components->pm != NULL) profilemanager_delete_manager(components->pm);
 	
-	// Initialized connections
-	g_list_foreach(components->initialized_connections, (GFunc)gems_clear_gems_connection,NULL);
-	g_list_free(components->initialized_connections);
-	components->initialized_connections = NULL;
-	
-	// Error list
-	g_list_foreach(components->connection_errorlist,(GFunc)gems_clear_connection_error,NULL);
-	g_list_free(components->connection_errorlist);
-	components->connection_errorlist = NULL;
+	// Clean lists
+	jammo_gems_cleanup_lists();
 	
 	// Clear communication
 	gems_communication_cleanup();
@@ -255,6 +251,10 @@ void jammo_gems_cleanup()
 	if(components->service_group->enabled == TRUE) ph_c_unregister_service(components->ph,GROUP_SERVICE_NAME);
 	gems_service_group_cleanup();
 	
+	// Clear Collaboration service related
+	if(components->service_group->enabled == TRUE) ph_c_unregister_service(components->ph,COLLABORATION_SERVICE_NAME);
+	gems_service_collaboration_cleanup();
+	
 	// delete peerhood
 	if(components->ph != NULL)
 	{
@@ -270,14 +270,35 @@ void jammo_gems_cleanup()
 	}
 	
 	// Free the structure
-	g_free(components);
-	components = NULL;
+	//g_free(components);
+	//components = NULL;
+	gems_free_data();
 	
 	logmsg = g_strdup_printf("jammo_gems_cleanup: done");
 	cem_add_to_log(logmsg,LOG_DEBUG);
 	g_free(logmsg);
 }
 
+void jammo_gems_cleanup_lists()
+{
+	gems_components* components = gems_get_data();
+	
+	if(components->initialized_connections != NULL)
+	{
+		// Initialized connections
+		g_list_foreach(components->initialized_connections, (GFunc)gems_clear_gems_connection,NULL);
+		g_list_free(components->initialized_connections);
+		components->initialized_connections = NULL;
+	}
+	
+	if(components->connection_errorlist != NULL)
+	{
+		// Error list
+		g_list_foreach(components->connection_errorlist,(GFunc)gems_clear_connection_error,NULL);
+		g_list_free(components->connection_errorlist);
+		components->connection_errorlist = NULL;
+	}
+}
 
 void gems_list_profile_parameters()
 {
@@ -763,3 +784,103 @@ void gems_debugging_show_jammos()
 	}
 	else printf("No connected JamMos\n");
 }
+
+// Initialize PeerHood callback
+void gems_init_peerhood_callback()
+{
+	void (*notify)(short, const char*,void*) = NULL;
+	notify = &gems_ph_callback_notify;
+
+	void (*newconnection)(const unsigned short, MAbstractConnection*, int,void*) = NULL;
+	newconnection = &gems_ph_callback_newconnection;
+	
+	gems_get_data()->ph_cb = ph_c_create_callback(notify, newconnection,(void*)gems_get_data(),(void*)gems_get_data());
+	
+	gchar* logmsg = g_strdup_printf("gems_networking_init_peerhood_callback: new callback created.");
+	cem_add_to_log(logmsg,LOG_INFO);
+	g_free(logmsg);
+}
+
+/* Enable network timeout functions
+ * 
+ * Add all timeout functions related to networked components inside this function.
+ * This will be called if connection to PeerHood is lost and it is found again.
+ */
+void gems_enable_network_timeout_functions()
+{
+	g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE,200,(GSourceFunc)gems_process_connections, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE,200,(GSourceFunc)gems_service_jammo_process_connections, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_LOW,200,(GSourceFunc)gems_communication_search_jammos, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_LOW,300,(GSourceFunc)gems_communication_retrieve_profiles, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_DEFAULT,100,(GSourceFunc)gems_communication_process_connections, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_LOW,200,(GSourceFunc)gems_communication_request_groups, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_LOW,350,(GSourceFunc)gems_communication_sanitize_grouplist, NULL, NULL);
+}
+
+void gems_register_services()
+{
+	gems_components* components = gems_get_data();
+	gchar* logmsg = NULL;
+	gint serviceport = 0;
+	
+	// Create JamMo service
+	if((serviceport = ph_c_register_service(components->ph,JAMMO_SERVICE_NAME,"application:jammo,method:versioncheck")) != 0)
+	{
+		components->service_jammo->enabled = TRUE; //enabled
+		components->service_jammo->port = serviceport;
+		logmsg = g_strdup_printf("gems_networking_register_services: registered %s service.",JAMMO_SERVICE_NAME);
+	}
+	else
+	{
+		components->service_jammo->enabled = FALSE;
+		logmsg = g_strdup_printf("gems_networking_register_services: cannot register %s service.",JAMMO_SERVICE_NAME);
+	}
+	cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+	g_free(logmsg);
+	
+	// Create JamMo service
+	if((serviceport = ph_c_register_service(components->ph,PROFILE_SERVICE_NAME,"application:jammo,method:getprofile")) != 0)
+	{
+		components->service_profile->enabled = TRUE; //enabled
+		components->service_profile->port = serviceport;
+		logmsg = g_strdup_printf("gems_networking_register_services: registered %s service.",PROFILE_SERVICE_NAME);
+	}
+	else
+	{
+		components->service_profile->enabled = FALSE;
+		logmsg = g_strdup_printf("gems_networking_register_services: cannot register %s service.",PROFILE_SERVICE_NAME);
+	}
+	cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+	g_free(logmsg);
+	
+	// Create JamMo service
+	if((serviceport = ph_c_register_service(components->ph,GROUP_SERVICE_NAME,"application:jammo,method:groupmanager")) != 0)
+	{
+		components->service_group->enabled = TRUE; //enabled
+		components->service_group->port = serviceport;
+		logmsg = g_strdup_printf("gems_networking_register_services: registered %s service.",GROUP_SERVICE_NAME);
+	}
+	else
+	{
+		components->service_group->enabled = FALSE;
+		logmsg = g_strdup_printf("gems_networking_register_services: cannot register %s service.",GROUP_SERVICE_NAME);
+	}
+	cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+	g_free(logmsg);
+
+	// Create Collaboration service
+	if((serviceport = ph_c_register_service(components->ph,COLLABORATION_SERVICE_NAME,"application:jammo,method:collaboration")) != 0)
+	{
+		components->service_collaboration->enabled = TRUE; //enabled
+		components->service_collaboration->port = serviceport;
+		logmsg = g_strdup_printf("gems_networking_register_services: registered %s service.",COLLABORATION_SERVICE_NAME);
+	}
+	else
+	{
+		components->service_collaboration->enabled = FALSE;
+		logmsg = g_strdup_printf("gems_networking_register_services: cannot register %s service.",COLLABORATION_SERVICE_NAME);
+	}
+	cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+	g_free(logmsg);
+
+}
