@@ -877,17 +877,27 @@ void gems_service_group_cleanup()
 	if(data != NULL)
 	{
 		if(data->group_info != NULL) gems_clear_group_info(data->group_info);
-		if(data->other_groups != NULL)
-		{
-			g_list_foreach(data->other_groups,(GFunc)gems_clear_group_info,NULL);
-			g_list_free(data->other_groups);
-			data->other_groups = NULL;
-		}
+		
+		gems_service_group_cleanup_lists();
+		
 		g_free(data);
 		data = NULL;
 	}
 }
 
+void gems_service_group_cleanup_lists()
+{
+	gems_service_group* data = gems_get_data()->service_group;
+
+	if(data->other_groups != NULL)
+	{
+		g_list_foreach(data->other_groups,(GFunc)gems_clear_group_info,NULL);
+		g_list_free(data->other_groups);
+		data->other_groups = NULL;
+	}
+}
+
+
 void gems_service_group_handle_unlocked(gems_components* data, gems_connection* element)
 {
 	if(gems_group_active(NULL)) // group is active and we're in it
