@@ -7,6 +7,7 @@ License:LGPL 2.1
 
 #include <gst/gst.h>
 #include <gst/base/gstbasesrc.h>
+#include "../../jammo-midi.h"
 
 G_BEGIN_DECLS
 
@@ -44,8 +45,7 @@ typedef float (*JammoSampleInstrumentFunc) (gdouble);
 /*If you change these:
 -generate freq-table again
 -check names of notes (it is array with size 12) */
-#define NUMBER_OF_OCTAVES 8
-#define NUMBER_OF_NOTES 12
+#define MAXNOTES 20
 
 typedef struct _SlicedWav SlicedWav;
 
@@ -82,11 +82,13 @@ gint needed_pitch;	//if base was is D#4, put -1 if you want D4. put +1 if you wa
 };
 
 //Notes are ints too.
-typedef struct _JammoMidiEvent {
+/*typedef struct _JammoMidiEvent {
 	int note;
 	int octave;
 	GstClockTime timestamp;
+	int type; //0=start, 1=stop
 } JammoMidiEvent;
+*/
 
 /**
  * GstJammoSampler:
@@ -95,18 +97,20 @@ typedef struct _JammoMidiEvent {
 struct _GstJammoSampler {
   GstBaseSrc parent;
 
-  GstClockTime playback_starting_time; //Starting point for playback
+  //GstClockTime playback_starting_time; //Starting point for playback
   int playback_index_for_event;  //This is index for next event in eventlist
   gboolean playback_not_started; 
   GstClockTime creation_time;
 
+	GstClockTime time;
+
   JammoSamplerProcessFunc process;
 
   GList* eventlist;
   gboolean recording;
   /* parameters */
   gdouble volume;
-  gboolean note_table[NUMBER_OF_OCTAVES*NUMBER_OF_NOTES];
+  gboolean note_table[JAMMOMIDI_NUMBER_OF_NOTES];
 
   /* Looping, non-looping or one-shot */
   GstJammoSamplerSampleCategory category;
@@ -126,16 +130,17 @@ struct _GstJammoSampler {
   gint64 sample_stop;
   gboolean check_seek_stop;
   gboolean eos_reached;
+	gboolean last_event_processed;
   gint generate_samples_per_buffer;	/* used to generate a partial buffer */
   gboolean can_activate_pull;
   
   /* waveform specific context data */
-  gdouble accumulators[NUMBER_OF_OCTAVES*NUMBER_OF_NOTES];			/* phase angle */
+  gdouble accumulators[JAMMOMIDI_NUMBER_OF_NOTES];			/* phase angle */
   gint notes_playing;  		/*How many notes are playing in this moment*/
 
   gchar *base_folder; /*This folder of virtual instrument samples*/
 
-  SlicedWav *wav_based_instrument[NUMBER_OF_OCTAVES*NUMBER_OF_NOTES];	
+  SlicedWav *wav_based_instrument[JAMMOMIDI_NUMBER_OF_NOTES];	
 };
 
 struct _GstJammoSamplerClass {
