@@ -7,6 +7,9 @@ License:LGPL 2.1
 
 #include <gst/gst.h>
 #include <gst/base/gstbasesrc.h>
+#include <sndfile.h>
+#include "../../jammo-slider-event.h"
+
 
 G_BEGIN_DECLS
 
@@ -42,6 +45,10 @@ typedef gfloat (*JammoSliderInstrumentFunc) (gdouble);
 #define NUMBER_OF_OCTAVES 8
 #define NUMBER_OF_NOTES 12
 
+// maximum and minimum frequencies slider tries to produce
+#define MINFREQ 10.0
+#define MAXFREQ 2000.0
+
 /**
  * GstJammoSlider:
  * object structure.
@@ -74,27 +81,41 @@ struct _GstJammoSlider {
   gboolean eos_reached;
   gint generate_samples_per_buffer;	/* used to generate a partial buffer */
   gboolean can_activate_pull;
+
+	GstClockTime time; /* used for playback and timestamps */
+	gboolean recording;
+	GList * eventlist;
+	gint playback_index_for_event;
+	gboolean last_event_processed;
+	JammoSliderEvent previous_event;
+	GstClockTime time_of_last_stored_event;
   
   /* waveform specific context data */
-  gdouble accumulators[NUMBER_OF_OCTAVES*NUMBER_OF_NOTES];			/* phase angle */
   gfloat tremolo_phase;		/*Vibration of tones*/
-  gfloat slider_freq;
-  gfloat slider_freq_slow;
-  gfloat slider_accumulator;
-  gboolean slider_state;
-  gint slider_rounding_counter;
-  gboolean slider_first_freq;
+  gfloat freq;
+  gfloat freq_slow;
+  gfloat accumulator;
+  gboolean state;
+  gint rounding_counter;
+  gboolean released;
+	gboolean rounded;
+  gint release_counter;
+  gfloat release_multiplier;
 
   /* Slider ADSR */
   gint attack;
   gint decay;
   gfloat sustain;
   gint release;
-
   gint adsr_counter;
-  gboolean released;
-  gint release_counter;
-  gfloat release_multiplier;
+  
+  /* blow sound for flute like instrument */
+	SNDFILE * blow_file;
+	SF_INFO blow_file_info;
+  gfloat blow_accumulator;
+  gint blow_loop_start;
+  gint blow_loop_end;
+  gint16 * blow_table;
 			
 };
 
