@@ -12,6 +12,7 @@
 #include "jammo-meam.h"
 #include "jammo-meam-private.h"
 #include "jammo-pipeline.h"
+#include "jammo-instrument-track.h"
 
 /**
  * SECTION:jammo-sequencer
@@ -36,14 +37,21 @@ enum {
 enum {
 	STARTED,
 	STOPPED,
+	PAUSED,
 	LAST_SIGNAL
 };
 
 struct _JammoSequencerPrivate {
 	GstElement* pipeline;
 	GstElement* adder;
+	GstElement* sink;
+	GstElement* audioconvert;
+	GstElement* vorbisenc;
+	GstElement* oggmux;
+	GstElement* filesink;
 	GList* tracks;
 	guint64 duration;
+	gint mode;
 	
 	guint ready : 1;
 	guint playing : 1;
@@ -54,6 +62,9 @@ static void on_duration_notify(GObject* object, GParamSpec* param_spec, gpointer
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+// quality for ogg file created in file mode
+#define OGGQUALITY 0.5
+
 /**
  * jammo_sequencer_new:
  *
@@ -119,6 +130,7 @@ void jammo_sequencer_stop(JammoSequencer* sequencer) {
  */
 void jammo_sequencer_pause(JammoSequencer* sequencer) {
 	gst_element_set_state(sequencer->priv->pipeline, GST_STATE_PAUSED);
+	g_signal_emit(sequencer, signals[PAUSED], 0);
 	sequencer->priv->playing = FALSE;
 }
 
@@ -241,6 +253,176 @@ guint64 jammo_sequencer_get_position(JammoSequencer* sequencer) {
 	return (guint64)position;
 }
 
+/**
+ * jammo_sequencer_set_position:
+ * @sequencer: a #JammoSequencer
+ * @position: position in nanoseconds
+ *
+ * Seeks the position of playing which is needed in real-time pair jamming.
+ *
+ * Return value: boolean success or not.
+ */
+
+gboolean jammo_sequencer_set_position(JammoSequencer* sequencer, guint64 position) {
+	gboolean success;
+
+	success = gst_element_seek(sequencer->priv->pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_ACCURATE | GST_SEEK_FLAG_FLUSH,
+	                           GST_SEEK_TYPE_SET, (gint64)position, GST_SEEK_TYPE_SET, GST_CLOCK_TIME_NONE);
+
+	return success;
+}
+
+/**
+ * jammo_sequencer_file_mode:
+ * @sequencer: a #JammoSequencer
+ *
+ * Sets sequencer in file mode meaning that instead of playing it generates an ogg file.
+ *
+ * Return value: int 0 success, 1 fail.
+ */
+gint jammo_sequencer_file_mode(JammoSequencer* sequencer, gchar* filename) {
+	// no need to do anything if already in file mode
+	if (sequencer->priv->mode==SEQUENCER_FILE_MODE) {
+		return 0;
+	}
+
+	// stop sequencer just in case
+	jammo_sequencer_stop(sequencer);
+
+	// switch ready to false
+	sequencer->priv->ready = FALSE;
+
+	// create wavenc and filesink
+	if (!(sequencer->priv->audioconvert = gst_element_factory_make("audioconvert", NULL))) {
+		g_warning("Could not create GStreamer element 'audioconvert'.\n");
+		return 1;
+	}	else if (!(sequencer->priv->vorbisenc = gst_element_factory_make("vorbisenc", NULL))) {
+		g_warning("Could not create GStreamer element 'vorbisenc'.\n");
+		return 1;
+	} else if (!(sequencer->priv->oggmux = gst_element_factory_make("oggmux", NULL))) {
+		g_warning("Could not create GStreamer element 'oggmux'.\n");
+		return 1;
+	} else if (!(sequencer->priv->filesink = gst_element_factory_make("filesink", NULL))) {
+		g_warning("Could not create GStreamer element 'filesink'.\n");
+		return 1;
+	}
+
+	// set quality to vorbisenc
+	g_object_set(G_OBJECT(sequencer->priv->vorbisenc), "quality", OGGQUALITY, NULL);
+
+	// set filename to filesink
+	g_object_set(G_OBJECT(sequencer->priv->filesink), "location", filename, NULL);
+
+	// unlink and remove audiosink
+	gst_element_unlink(sequencer->priv->adder, sequencer->priv->sink);
+	gst_bin_remove(GST_BIN(sequencer->priv->pipeline), sequencer->priv->sink);
+
+	// add audioconvert, vorbisenc, oggmux and filesink to bin
+	gst_bin_add_many(GST_BIN(sequencer->priv->pipeline), sequencer->priv->audioconvert, sequencer->priv->vorbisenc, sequencer->priv->oggmux, sequencer->priv->filesink, NULL);
+
+	// link audioconvert, vorbisenc, oggmux and filesink
+	if (!gst_element_link(sequencer->priv->adder, sequencer->priv->audioconvert)) {
+		g_warning("linking adder and audioconvert failed");
+		return 1;
+	}
+	if (!gst_element_link(sequencer->priv->audioconvert, sequencer->priv->vorbisenc)) {
+		g_warning("linking audioconvert and vorbisenc failed");
+		return 1;
+	}
+	if (!gst_element_link(sequencer->priv->vorbisenc, sequencer->priv->oggmux)) {
+		g_warning("linking vorbisenc and oggmux failed");
+		return 1;
+	}
+	if (!gst_element_link(sequencer->priv->oggmux, sequencer->priv->filesink)) {
+		g_warning("linking oggmux and filesink failed");
+		return 1;
+	}
+
+	// set all instruments as not live & realtime
+	GList * list;
+	for (list = sequencer->priv->tracks; list; list = list->next) {
+		if(JAMMO_IS_INSTRUMENT_TRACK(list->data)) {
+			jammo_instrument_track_set_live(JAMMO_INSTRUMENT_TRACK(list->data), FALSE);
+			jammo_instrument_track_set_realtime(JAMMO_INSTRUMENT_TRACK(list->data), FALSE);
+		}
+	}
+
+	sequencer->priv->mode=SEQUENCER_FILE_MODE;
+
+	// switch ready to true
+	sequencer->priv->ready = TRUE;
+
+	return 0;
+}
+
+/**
+ * jammo_sequencer_audio_mode:
+ * @sequencer: a #JammoSequencer
+ *
+ * Sets sequencer back in audio mode meaning that instead of generating
+ * a wav file it plays audio
+ *
+ * Return value: int 0 success, 1 fail.
+ */
+gint jammo_sequencer_audio_mode(JammoSequencer* sequencer) {
+	// no need to do anything if already in audio mode
+	if (sequencer->priv->mode==SEQUENCER_AUDIO_MODE) {
+		return 0;
+	}
+
+	// stop sequencer just in case
+	jammo_sequencer_stop(sequencer);
+
+	// switch ready to false
+	sequencer->priv->ready = FALSE;
+
+	// create sink
+	if (!(sequencer->priv->sink = gst_element_factory_make("autoaudiosink", NULL))) {
+		g_warning("Could not create GStreamer element 'wavenc'.\n");
+		return 1;
+	}
+
+	// unlink, audioconvert, vorbisenc, oggmux and filesink
+	gst_element_unlink(sequencer->priv->adder, sequencer->priv->audioconvert);
+	gst_element_unlink(sequencer->priv->audioconvert, sequencer->priv->vorbisenc);
+	gst_element_unlink(sequencer->priv->vorbisenc, sequencer->priv->oggmux);
+	gst_element_unlink(sequencer->priv->oggmux, sequencer->priv->filesink);
+
+	// remove audioconvert, vorbisenc, oggmux and filesink from bin
+	gst_bin_remove(GST_BIN(sequencer->priv->pipeline), sequencer->priv->audioconvert);
+	gst_bin_remove(GST_BIN(sequencer->priv->pipeline), sequencer->priv->vorbisenc);
+	gst_bin_remove(GST_BIN(sequencer->priv->pipeline), sequencer->priv->oggmux);
+	gst_bin_remove(GST_BIN(sequencer->priv->pipeline), sequencer->priv->filesink);
+
+	// add sink to bin
+	if (!gst_bin_add(GST_BIN(sequencer->priv->pipeline), sequencer->priv->sink)) {
+		g_warning("linking sink failed\n");
+		return 1;
+	}
+
+	// link sink
+	if (!gst_element_link(sequencer->priv->adder, sequencer->priv->sink)) {
+		g_warning("linking sink failed\n");
+		return 1;
+	}
+
+	GList * list;
+	// set all instruments as live and realtime
+	for (list = sequencer->priv->tracks; list; list = list->next) {
+		if(JAMMO_IS_INSTRUMENT_TRACK(list->data)) {
+			jammo_instrument_track_set_live(JAMMO_INSTRUMENT_TRACK(list->data), TRUE);
+			jammo_instrument_track_set_realtime(JAMMO_INSTRUMENT_TRACK(list->data), TRUE);
+		}
+	}
+
+	sequencer->priv->mode=SEQUENCER_AUDIO_MODE;
+
+	// switch ready to true
+	sequencer->priv->ready = TRUE;
+
+	return 0;
+}
+
 static void jammo_sequencer_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	JammoSequencer* sequencer;
 	
@@ -324,13 +506,23 @@ static void jammo_sequencer_class_init(JammoSequencerClass* klass) {
 					       NULL, NULL,
 					       g_cclosure_marshal_VOID__VOID,
 					       G_TYPE_NONE, 0);
+	/**
+	 * JammoSequencer::paused:
+	 * @sequencer: the object which received the signal
+	 *
+	 * The ::paused signal is emitted when the playing (of all tracks) has paused.
+	 */
+	signals[PAUSED] = g_signal_new("paused", G_TYPE_FROM_CLASS(gobject_class),
+					       G_SIGNAL_RUN_LAST, 0,
+					       NULL, NULL,
+					       g_cclosure_marshal_VOID__VOID,
+					       G_TYPE_NONE, 0);
 
 	g_type_class_add_private(gobject_class, sizeof(JammoSequencerPrivate));
 }
 
 static void jammo_sequencer_init(JammoSequencer* sequencer) {
 	GstBus* bus;
-	GstElement* sink;
 
 	sequencer->priv = G_TYPE_INSTANCE_GET_PRIVATE(sequencer, JAMMO_TYPE_SEQUENCER, JammoSequencerPrivate);
 
@@ -343,12 +535,13 @@ static void jammo_sequencer_init(JammoSequencer* sequencer) {
 
 	if (!(sequencer->priv->adder = gst_element_factory_make("adder", NULL))) {
 		g_warning("Could not create GStreamer element 'adder'.\n");
-	} else if (!(sink = gst_element_factory_make("autoaudiosink", NULL))) {
-		g_warning("Could not create GStreamer element 'autoaudiosink'.\n");	
+	} else if (!(sequencer->priv->sink = gst_element_factory_make("autoaudiosink", NULL))) {
+		g_warning("Could not create GStreamer element 'autoaudiosink'.\n");
 	} else {
-		gst_bin_add_many(GST_BIN(sequencer->priv->pipeline), sequencer->priv->adder, sink, NULL);
-		gst_element_link(sequencer->priv->adder, sink);
+		gst_bin_add_many(GST_BIN(sequencer->priv->pipeline), sequencer->priv->adder, /*sequencer->priv->volume,*/ sequencer->priv->sink, NULL);
+		gst_element_link(sequencer->priv->adder, sequencer->priv->sink);
 	}
+	sequencer->priv->mode=SEQUENCER_AUDIO_MODE;
 }
 
 static gboolean on_bus_message(GstBus* bus, GstMessage* message, gpointer data) {
